// src/mysql.ts
import { Session } from "@shopify/shopify-api";

// src/migrations.ts
import {
  MigrationOperation
} from "@shopify/shopify-app-session-storage";
var migrationList = [
  new MigrationOperation(
    "migrateScopeFieldToVarchar1024",
    migrateScopeFieldToVarchar1024
  )
];
async function migrateScopeFieldToVarchar1024(connection) {
  await connection.query(`ALTER TABLE ${connection.sessionStorageIdentifier} 
      MODIFY COLUMN scope varchar(1024)`);
}

// src/mysql-connection.ts
import * as mysql from "mysql2/promise";
var MySqlConnection = class {
  sessionStorageIdentifier;
  ready;
  connection;
  constructor(sql_config, sessionStorageIdentifier) {
    this.ready = this.init(sql_config);
    this.sessionStorageIdentifier = sessionStorageIdentifier;
  }
  async query(query, params = []) {
    await this.ready;
    return this.connection.query(query, params);
  }
  async connect() {
    await this.ready;
    return Promise.resolve();
  }
  async disconnect() {
    await this.ready;
    await this.connection.end();
  }
  getDatabase() {
    return this.connection.config.database;
  }
  async hasTable(tablename) {
    await this.ready;
    const query = `
      SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES 
        WHERE TABLE_NAME = ${this.getArgumentPlaceholder()} 
          AND TABLE_SCHEMA = ${this.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [
      tablename,
      this.getDatabase()
    ]);
    return Array.isArray(rows) && rows.length === 1;
  }
  getArgumentPlaceholder(_) {
    return `?`;
  }
  async init(sql_config) {
    this.connection = await mysql.createConnection({
      user: sql_config.username,
      password: sql_config.password,
      host: sql_config.host,
      database: sql_config.dbName
    });
  }
};

// src/mysql-migrator.ts
import {
  RdbmsSessionStorageMigrator
} from "@shopify/shopify-app-session-storage";
var MySqlSessionStorageMigrator = class extends RdbmsSessionStorageMigrator {
  constructor(dbConnection, opts = {}) {
    super(dbConnection, opts);
  }
  async initMigrationPersistence() {
    const discardCreateTable = await this.connection.hasTable(
      this.options.migrationDBIdentifier
    );
    if (!discardCreateTable) {
      const migration = `
      CREATE TABLE ${this.options.migrationDBIdentifier} (
        ${this.getOptions().migrationNameColumnName} varchar(255) NOT NULL PRIMARY KEY
    );`;
      await this.connection.query(migration, []);
    }
  }
};

// src/mysql.ts
var defaultMySQLSessionStorageOptions = {
  sessionTableName: "shopify_sessions",
  migratorOptions: {
    migrationDBIdentifier: "shopify_sessions_migrations",
    migrationNameColumnName: "migration_name",
    migrations: migrationList
  }
};
var MySQLSessionStorage = class {
  static withCredentials(sql_config, opts) {
    return new MySQLSessionStorage(
      sql_config,
      opts
    );
  }
  ready;
  internalInit;
  options;
  connection;
  migrator;
  constructor(sql_config, opts = {}) {
    this.options = { ...defaultMySQLSessionStorageOptions, ...opts };
    this.internalInit = this.init(sql_config);
    this.ready = this.initMigrator(this.options.migratorOptions);
  }
  async storeSession(session) {
    await this.ready;
    const entries = session.toPropertyArray().map(
      ([key, value]) => key === "expires" ? [key, Math.floor(value / 1e3)] : [key, value]
    );
    const query = `
      REPLACE INTO ${this.options.sessionTableName}
      (${entries.map(([key]) => key).join(", ")})
      VALUES (${entries.map(() => `${this.connection.getArgumentPlaceholder()}`).join(", ")})
    `;
    await this.connection.query(
      query,
      entries.map(([_key, value]) => value)
    );
    return true;
  }
  async loadSession(id) {
    await this.ready;
    const query = `
      SELECT * FROM \`${this.options.sessionTableName}\`
      WHERE id = ${this.connection.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [id]);
    if (!Array.isArray(rows) || rows?.length !== 1)
      return void 0;
    const rawResult = rows[0];
    return this.databaseRowToSession(rawResult);
  }
  async deleteSession(id) {
    await this.ready;
    const query = `
      DELETE FROM ${this.options.sessionTableName}
      WHERE id = ${this.connection.getArgumentPlaceholder()};
    `;
    await this.connection.query(query, [id]);
    return true;
  }
  async deleteSessions(ids) {
    await this.ready;
    const query = `
      DELETE FROM ${this.options.sessionTableName}
      WHERE id IN (${ids.map(() => `${this.connection.getArgumentPlaceholder()}`).join(",")});
    `;
    await this.connection.query(query, ids);
    return true;
  }
  async findSessionsByShop(shop) {
    await this.ready;
    const query = `
      SELECT * FROM ${this.options.sessionTableName}
      WHERE shop = ${this.connection.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [shop]);
    if (!Array.isArray(rows) || rows?.length === 0)
      return [];
    const results = rows.map((row) => {
      return this.databaseRowToSession(row);
    });
    return results;
  }
  async disconnect() {
    await this.connection.disconnect();
  }
  async init(sql_config) {
    this.connection = new MySqlConnection(sql_config, this.options.sessionTableName);
    await this.createTable();
  }
  async createTable() {
    const hasSessionTable = await this.connection.hasTable(
      this.options.sessionTableName
    );
    if (!hasSessionTable) {
      const query = `
        CREATE TABLE ${this.options.sessionTableName} (
          id varchar(255) NOT NULL PRIMARY KEY,
          shop varchar(255) NOT NULL,
          state varchar(255) NOT NULL,
          isOnline tinyint NOT NULL,
          scope varchar(255),
          expires integer,
          onlineAccessInfo varchar(255),
          accessToken varchar(255)
        )
      `;
      await this.connection.query(query);
    }
  }
  databaseRowToSession(row) {
    if (row.expires)
      row.expires *= 1e3;
    return Session.fromPropertyArray(Object.entries(row));
  }
  async initMigrator(migratorOptions) {
    await this.internalInit;
    if (migratorOptions === null) {
      return Promise.resolve();
    } else {
      this.migrator = new MySqlSessionStorageMigrator(
        this.connection,
        migratorOptions
      );
      this.migrator.validateMigrationList(migrationList);
      return this.migrator.applyMigrations();
    }
  }
};
export {
  MySQLSessionStorage
};
//# sourceMappingURL=index.js.map
